package Class::Default;

# The Class::Default package allows a class that inherits from
# it to act as both an instantiatable and static class.

# See POD for more details.

require 5.005;
use strict;
use Carp;

# Define globals
use vars qw{$VERSION %DEFAULT};
BEGIN { 
	$VERSION = '1.0';

	# Create the default object storage.
	%DEFAULT = ();
}

# Get the default object if we are passed the class name.
sub _self { 
	my $either = shift;
	ref($either) ? $either 
	: $DEFAULT{$either} 
		|| ($DEFAULT{$either} = $either->_create_default_object)
		|| croak "Error while creating default object";
}

# Suplimentary method to reliably get ONLY the class
sub _class { ref $_[0] or $_[0] }

# Retrieve the default object for a class, either from
# the cache, or create it new.
sub _get_default {
	my $class = shift; 
	$DEFAULT{$class} 
		|| ($DEFAULT{$class} = $class->_create_default_object)
		|| croak "Error while creating default object";
}

# Creates the default object. 
# Used to provide options to a constructor to create the default object.
sub _create_default_object {
	my $class = shift;

	### When you copy this to overload it, you should add
	### arguments to the constructor call as needed.

	# Create the new object.
	my $self = $class->new;

	### Make any modifications to the default object here

	$self;
}

1;
